/* $Id$

This file is part of libmspgl
Copyright © 2007  Mikko Rasa, Mikkosoft Productions
Distributed under the LGPL
*/

#include "except.h"
#include "light.h"
#include "misc.h"

using namespace std;

namespace Msp {
namespace GL {

Light::Light():
	ambient(0),
	diffuse(1),
	specular(1),
	x(0), y(0), z(1), w(0),
	sdx(0), sdy(0), sdz(-1),
	spot_exp(0),
	spot_cutoff(180)
{ }

void Light::set_ambient(const Color &c)
{
	ambient=c;
}

void Light::set_diffuse(const Color &c)
{
	diffuse=c;
}

void Light::set_specular(const Color &c)
{
	specular=c;
}

void Light::set_position(float x_, float y_, float z_, float w_)
{
	x=x_;
	y=y_;
	z=z_;
	w=w_;
}

void Light::bind() const
{
	if(current_lights[current_unit]!=this)
	{
		GLenum l=GL_LIGHT0+current_unit;
		enable(l);
		glLightfv(l, GL_AMBIENT, &ambient.r);
		glLightfv(l, GL_DIFFUSE, &diffuse.r);
		glLightfv(l, GL_SPECULAR, &specular.r);
		glLightfv(l, GL_POSITION, &x);
		current_lights[current_unit]=this;
	}
}

void Light::bind_to(unsigned i) const
{
	activate(i);
	bind();
}

void Light::activate(unsigned i)
{
	static unsigned max_lights=get_i(GL_MAX_LIGHTS);

	if(i>=max_lights)
		throw InvalidParameterValue("Light unit index out of range");

	if(i>=current_lights.size())
		current_lights.resize(i+1);

	current_unit=i;
}

void Light::unbind()
{
	if(current_lights[current_unit])
	{
		disable(GL_LIGHT0+current_unit);
		current_lights[current_unit]=0;
	}
}

unsigned Light::current_unit=0;
vector<const Light *> Light::current_lights(1);

} // namespace GL
} // namespace Msp
